@@ -10,6 +10,17 @@
 *   02/08/05   DGP   Added FAP support and multiple assembly.
 *   03/08/05   DGP   Added CPU model.
 *   03/25/05   DGP   Added 704 model and instructions.
+*   06/07/05   DGP   Added CTSS insts and RMT and TAPENO psops.
+*   08/19/05   SCM   Added MSYS ifdefs.
+*   12/15/05   RMS   MINGW changes.
+*   07/30/07   DGP   Change mktemp to mkstemp.
+*   09/30/09   DGP   Added FAP Linkage Director.
+*   04/30/10   DGP   Added sequence checking option.
+*   05/10/10   DGP   Added opcore ACORE/BCORE support.
+*   06/07/10   DGP   Added punch symbol table option.
+*   06/15/10   DGP   Added XBUILD sym def for CTSS.
+*   11/01/10   DGP   Added CTSS Common mode.
+*   03/30/11   DGP   Allow multiple include directories.
 *	
 ***********************************************************************/
 
@@ -29,6 +40,8 @@ int absolute;			/* In absolute section */
 int absmod;			/* In absolute module */
 int genxref;			/* Generate cross reference listing */
 int widemode;			/* Generate wide listing */
+int sequcheck;			/* Check sequence numbers */
+int lastsequ;			/* Last sequence number */
 int monsym;			/* Include IBSYS Symbols (MONSYM) */
 int jobsym;			/* Include IBJOB Symbols (JOBSYM) */
 int addext;			/* Add extern for undef'd symbols (!absolute) */
@@ -45,7 +58,9 @@ int opdefcount;			/* Number of user defined opcode in opdef */
 int begincount;			/* Number of BEGINs */
 int inpass;			/* Which pass are we in */
 int errcount;			/* Number of errors in assembly */
+int warncount;			/* Number of warnings in assembly */
 int errnum;			/* Index into errline array */
+int warnnum;			/* Index into warnline array */
 int linenum;			/* Source line number */
 int p1errcnt;			/* Number of pass 0/1 errors */
 int p1erralloc;			/* Number of pass 0/1 errors allocated */
@@ -56,16 +71,29 @@ int litorigin;			/* Literal pool origin */
 int litpool;			/* Literal pool size */
 int headcount;			/* Number of entries in headtable */
 int cpumodel;			/* CPU model (709, 7090 7094) */
+int filenum;			/* INSERT file index */
+int fileseq;			/* INSERT file sequence */
+int genlnkdir;			/* FAP generate linkage director */
+int nolnkdir;			/* FAP DO NOT generate linkage director */
+int bcore;			/* BCORE assembly */
+int punchsymtable;		/* Punch symbol table */
+int ctsscommon;			/* Relocate CTSS Common mode */
+int commonctr;			/* Common counter */
+int commonused;			/* Common used */
+int incldircnt;			/* Include dir count */
 
 char errline[10][120];		/* Pass 2 error lines for current statment */
+char warnline[10][120];		/* Pass 2 warning lines for current statment */
 char inbuf[MAXLINE];		/* The input buffer for the scanners */
 char lbl[MAXLBLLEN+2];		/* The object label */
-char deckname[MAXSYMLEN+2];	/* The assembly deckname */
+char deckname[MAXLBLLEN+2];	/* The assembly deckname */
 char ttlbuf[TTLSIZE+2];		/* The assembly TTL buffer */
 char titlebuf[TTLSIZE+2];	/* The assembly TITLE buffer */
 char qualtable[MAXQUALS][MAXSYMLEN+2]; /* The QUALifier table */
 char headtable[MAXHEADS];	/* HEAD table */
 char datebuf[48];		/* Date/Time buffer for listing */
+char *incldir[MAXINCLUDES];	/* -I include directory */
+char lnkdirsym[MAXSYMLEN+2];	/* Linkage director symbol */
 
 SymNode *addextsym;		/* Added symbol for externals */
 SymNode *symbols[MAXSYMBOLS];	/* The Symbol table */
@@ -76,334 +104,21 @@ OpDefCode *freeops;		/* Reusable opdef nodes */
 OpDefCode *opdefcode[MAXDEFOPS];/* The user defined opcode table */
 ErrTable p1error[MAXERRORS];	/* The pass 0/1 error table */
 BeginTable begintable[MAXBEGINS];/* The USE/BEGIN table */
+INSERTfile files[MAXASMFILES];	/* INSERT file descriptors */
+RMTSeq *rmthead;		/* Pointer to start of remote sequences */
+RMTSeq *rmttail;		/* Pointer to end of remote sequences */
 time_t curtime;			/* Assembly time */
 struct tm *timeblk;		/* Assembly time */
+char *pscanbuf;			/* Pointer for tokenizers */
 
 static int verbose;		/* Verbose mode */
 static int gblabsmod;		/* In absolute module */
 static int gblmonsym;		/* Include IBSYS Symbols (MONSYM) */
 static int gbljobsym;		/* Include IBJOB Symbols (JOBSYM) */
 static int filecnt;		/* File process count */
+static int reccnt;		/* File record count */
 
-/*
-** IBSYS System nucleus defintions (MONSYM).
-*/
-
-static int mondefsset;
-static SysDefs mapmondefs[] =
-{
-   { "SYSTRA", "S.S", 000100 },
-   { "SYSDAT", "S.S", 000101 },
-   { "SYSCUR", "S.S", 000102 },
-   { "SYSRET", "S.S", 000103 },
-   { "SYSKEY", "S.S", 000104 },
-   { "SYSSWS", "S.S", 000105 },
-   { "SYSPOS", "S.S", 000106 },
-   { "SYSUNI", "S.S", 000107 },
-   { "SYSUBC", "S.S", 000110 },
-   { "SYSUAV", "S.S", 000111 },
-   { "SYSUCW", "S.S", 000112 },
-   { "SYSRPT", "S.S", 000113 },
-   { "SYSCEM", "S.S", 000114 },
-   { "SYSDMP", "S.S", 000115 },
-   { "SYSIOX", "S.S", 000116 },
-   { "SYSIDR", "S.S", 000117 },
-   { "SYSCOR", "S.S", 000120 },
-   { "SYSLDR", "S.S", 000121 },
-   { "SYSACC", "S.S", 000122 },
-   { "SYSPID", "S.S", 000123 },
-   { "SYSCYD", "S.S", 000124 },
-   { "SYSSLD", "S.S", 000126 },
-   { "SYSTCH", "S.S", 000127 },
-   { "SYSTWT", "S.S", 000131 },
-   { "SYSGET", "S.S", 000132 },
-   { "SYSJOB", "S.S", 000133 },
-   { ".CHEXI", "",    000134 },
-   { ".MODSW", "",    000135 },
-   /* SYSUNI table */
-   { "SYSLB1", "S.S", 000140 },
-   { "SYSLB2", "S.S", 000141 },
-   { "SYSLB3", "S.S", 000142 },
-   { "SYSLB4", "S.S", 000143 },
-   { "SYSCRD", "S.S", 000144 },
-   { "SYSPRT", "S.S", 000145 },
-   { "SYSPCH", "S.S", 000146 },
-   { "SYSOU1", "S.S", 000147 },
-   { "SYSOU2", "S.S", 000150 },
-   { "SYSIN1", "S.S", 000151 },
-   { "SYSIN2", "S.S", 000152 },
-   { "SYSPP1", "S.S", 000153 },
-   { "SYSPP2", "S.S", 000154 },
-   { "SYSCK1", "S.S", 000155 },
-   { "SYSCK2", "S.S", 000156 },
-   { "SYSUT1", "S.S", 000157 },
-   { "SYSUT2", "S.S", 000160 },
-   { "SYSUT3", "S.S", 000161 },
-   { "SYSUT4", "S.S", 000162 },
-   { "SYSUT5", "S.S", 000163 },
-   { "SYSUT6", "S.S", 000164 },
-   { "SYSUT7", "S.S", 000165 },
-   { "SYSUT8", "S.S", 000166 },
-   { "SYSUT9", "S.S", 000167 },
-   /* IOEX communications region */
-   { ".ACTV",  "",    000702 },
-   { ".NDSEL", "",    000704 },
-   { ".MWR",   "",    000706 },
-   { ".PUNCH", "",    000707 },
-   { ".ENBSW", "",    000710 },
-   { ".PAWS",  "",    000711 },
-   { ".PAUSE", "",    000712 },
-   { ".STOP",  "",    000713 },
-   { ".SYMUN", "",    000714 },
-   { ".DECVD", "",    000715 },
-   { ".DECVA", "",    000716 },
-   { ".CKWAT", "",    000717 },
-   { ".BCD5R", "",    000720 },
-   { ".BCD5X", "",    000721 },
-   { ".CVPRT", "",    000722 },
-   { ".STOPD", "",    000723 },
-   { ".CHXAC", "",    000724 },
-   { ".URRX",  "",    000725 },
-   { ".RCTX",  "",    000726 },
-   { ".RCHX",  "",    000727 },
-   { ".TCOX",  "",    000730 },
-   { ".TRCX",  "",    000731 },
-   { ".ETTX",  "",    000732 },
-   { ".TEFX",  "",    000733 },
-   { ".TRAPX", "",    000734 },
-   { ".TRAPS", "",    000735 },
-   { ".COMM",  "",    000736 },
-   { ".LTPOS", "",    000737 },
-   { ".IOXSI", "",    000740 },
-   { ".CHPSW", "",    000741 },
-   { ".TRPSW", "",    000742 },
-   { ".FDAMT", "",    000743 },
-   { ".SDCXI", "",    000744 },
-   { ".STCXI", "",    000745 },
-   { ".COMMD", "",    000746 },
-   { ".IBCDZ", "",    000747 },
-   { ".CHXSP", "",    000750 },
-   { ".BLKSW", "",    000751 },
-   { "SYSORG", "S.S", 002652 },
-   { "SYSEND", "S.S", 077777 },
-   { "",       "",    -1     }
-};
-
-/*
-** IBJOB defintions (JOBSYM), pulls in mapmondefs.
-*/
-
-static int jobdefsset;
-static SysDefs jobdefs[] =
-{
-   { "SYSLOC", "", 021234 },
-   { "SYSFAZ", "", 021235 },
-   { "IBJCOR", "", 021236 },
-   { "IBJDAT", "", 021237 },
-   { ".JLDAT", "", 021240 },
-   { ".JTYPE", "", 021242 },
-   { ".JLIN",  "", 021243 },
-   { ".JVER",  "", 021244 },
-   { ".JKAPU", "", 021245 },
-   { "SYSDSB", "", 021246 },
-   { ".FDPOS", "", 021250 },
-   { "SSTRA",  "", 021253 },
-   { "ACTION", "", 021254 },
-   { "JOBIN",  "", 021255 },
-   { "JOBOU",  "", 021256 },
-   { "JOBPP",  "", 021257 },
-   { "IOEDIT", "", 021260 },
-   { "JREEL",  "", 021261 },
-   { "SUBSP",  "", 021262 },
-   { "PUNCH",  "", 021263 },
-   { "SYSSHD", "", 021264 },
-   { "LILDMP", "", 021265 },
-   { "IBSLB",  "", 021266 },
-   { "PRSW",   "", 021267 },
-   { "DEFINE", "", 021347 },
-   { "JOIN",   "", 021351 },
-   { "ATTACH", "", 021353 },
-   { "CLOSE",  "", 021355 },
-   { "OPEN",   "", 021357 },
-   { "READ",   "", 021361 },
-   { "WRITE",  "", 021363 },
-   { "STASH",  "", 021365 },
-   { "SUBSYS", "", 021412 },
-   { "",       "", -1     }
-};
-
-/*
-** FAP SST IBSYS symbols.
-*/
-
-static int ifapdefsset;
-static SysDefs ifapmondefs[] =
-{
-   { "SYSTRA", "", 000100 },
-   { "SYSDAT", "", 000101 },
-   { "SYSCUR", "", 000102 },
-   { "SYSRET", "", 000103 },
-   { "SYSKEY", "", 000104 },
-   { "SYSSWS", "", 000105 },
-   { "SYSPOS", "", 000106 },
-   { "SYSUNI", "", 000107 },
-   { "SYSUBC", "", 000110 },
-   { "SYSUAV", "", 000111 },
-   { "SYSUCW", "", 000112 },
-   { "SYSRPT", "", 000113 },
-   { "SYSCEM", "", 000114 },
-   { "SYSDMP", "", 000115 },
-   { "SYSIOX", "", 000116 },
-   { "SYSIDR", "", 000117 },
-   { "SYSCOR", "", 000120 },
-   { "SYSLDR", "", 000121 },
-   { "SYSACC", "", 000122 },
-   { "SYSPID", "", 000123 },
-   { "SYSCYD", "", 000124 },
-   { "SYSSLD", "", 000126 },
-   { "SYSTCH", "", 000127 },
-   { "SYSTWT", "", 000131 },
-   { "SYSGET", "", 000132 },
-   { "SYSJOB", "", 000133 },
-   { ".CHEXI", "", 000134 },
-   { ".MODSW", "", 000135 },
-   /* SYSUNI table */
-   { "SYSLB1", "", 000140 },
-   { "SYSLB2", "", 000141 },
-   { "SYSLB3", "", 000142 },
-   { "SYSLB4", "", 000143 },
-   { "SYSCRD", "", 000144 },
-   { "SYSPRT", "", 000145 },
-   { "SYSPCH", "", 000146 },
-   { "SYSOU1", "", 000147 },
-   { "SYSOU2", "", 000150 },
-   { "SYSIN1", "", 000151 },
-   { "SYSIN2", "", 000152 },
-   { "SYSPP1", "", 000153 },
-   { "SYSPP2", "", 000154 },
-   { "SYSCK1", "", 000155 },
-   { "SYSCK2", "", 000156 },
-   { "SYSUT1", "", 000157 },
-   { "SYSUT2", "", 000160 },
-   { "SYSUT3", "", 000161 },
-   { "SYSUT4", "", 000162 },
-   /* IOEX communications region */
-   { "(ACTIV", "", 000702 },
-   { "(ACTVX", "", 000703 },
-   { "(NDATA", "", 000704 },
-   { "(NDSLX", "", 000705 },
-   { "(PROUT", "", 000706 },
-   { "(PUNCH", "", 000707 },
-   { "(ENBSW", "", 000710 },
-   { "(PAWSX", "", 000711 },
-   { "(PAUSE", "", 000712 },
-   { "(STOPX", "", 000713 },
-   { "(SYMUN", "", 000714 },
-   { "(DECVD", "", 000715 },
-   { "(DECVA", "", 000716 },
-   { "(CKWAT", "", 000717 },
-   { "(BCD5R", "", 000720 },
-   { "(BCD5X", "", 000721 },
-   { "(CVPRT", "", 000722 },
-   { "(STOPD", "", 000723 },
-   { "(CHXAC", "", 000724 },
-   { "(URRXI", "", 000725 },
-   { "(RCTXI", "", 000726 },
-   { "(RCHXI", "", 000727 },
-   { "(TCOXI", "", 000730 },
-   { "(TRCXI", "", 000731 },
-   { "(ETTXI", "", 000732 },
-   { "(TEFXI", "", 000733 },
-   { "(TRAPX", "", 000734 },
-   { "(TRAPS", "", 000735 },
-   { "(COMMM", "", 000736 },
-   { "(LTPOS", "", 000737 },
-   { "(IOXSI", "", 000740 },
-   { "(CHPSW", "", 000741 },
-   { "(TRPSW", "", 000742 },
-   { "(FDAMT", "", 000743 },
-   { "(SDCXI", "", 000744 },
-   { "(STCXI", "", 000745 },
-   { "(COMMD", "", 000746 },
-   { "(IBCDZ", "", 000747 },
-   { "(CHXSP", "", 000750 },
-   { "SYSORG", "", 003720 },
-   { "SYSEND", "", 077777 },
-   { "",       "", -1     }
-};
-
-/*
-** FAP SST FORTRAN symbols, pulls in ifapmoddefs.
-*/
-
-static int ffapdefsset;
-static SysDefs ffapmondefs[] =
-{
-   { "LIBTAP", "", 000001 },
-   { "SYSTAP", "", 000001 },
-   { "FINTAP", "", 000002 },
-   { "SINTAP", "", 000003 },
-   { "TINTAP", "", 000004 },
-   { "MINTAP", "", 000005 },
-   { "MLSTAP", "", 000006 },
-   { "MBNTAP", "", 000007 },
-   { "MCHTAP", "", 000010 },
-   { "SNPTAP", "", 000011 },
-   { "(SNAP)", "", 001000 },
-   { "FLAGBX", "", 003730 },
-   { "LINECT", "", 003731 },
-   { "DATEBX", "", 003732 },
-   { "(PCBK)", "", 003733 },
-   { "BOTTOM", "", 003734 },
-   { "(TPER)", "", 004000 },
-   { "(CHKU)", "", 010000 },
-   { "(SUNV)", "", 010002 },
-   { "(SUAV)", "", 010003 },
-   { "(SKBP)", "", 020000 },
-   { "(SKDP)", "", 020002 },
-   { "(SKBC)", "", 020010 },
-   { "(SKDC)", "", 020012 },
-   { "(RBNP)", "", 040000 },
-   { "(WBNP)", "", 040001 },
-   { "(RDNP)", "", 040002 },
-   { "(WDNP)", "", 040003 },
-   { "(RBEP)", "", 040004 },
-   { "(WEFP)", "", 040005 },
-   { "(RDEP)", "", 040006 },
-   { "(RBNC)", "", 040010 },
-   { "(WBNC)", "", 040011 },
-   { "(RDNC)", "", 040012 },
-   { "(WDNC)", "", 040013 },
-   { "(RBEC)", "", 040014 },
-   { "(WEFC)", "", 040015 },
-   { "(RDEC)", "", 040016 },
-   { "(WROW)", "", 040031 },
-   { "BOTIOP", "", 073600 },
-   { "(LOAD)", "", 073600 },
-   { "(DIAG)", "", 073603 },
-   { "(TAPE)", "", 073606 },
-   { "(PRNT)", "", 073611 },
-   { "(PNCH)", "", 073614 },
-   { "(STAT)", "", 073617 },
-   { "(REST)", "", 073622 },
-   { "(STDN)", "", 073626 },
-   { "(SECL)", "", 073643 },
-   { "(MECL)", "", 073644 },
-   { "(DGLD)", "", 073645 },
-   { "(PGCT)", "", 073651 },
-   { "(LIBT)", "", 073652 },
-   { "(FGBX)", "", 073660 },
-   { "(LNCT)", "", 073661 },
-   { "(DATE)", "", 073662 },
-   { "(SNCT)", "", 073663 },
-   { "(MSLN)", "", 073664 },
-   { "(ENDS)", "", 073665 },
-   { "(LODR)", "", 073666 },
-   { "(LBLD)", "", 073667 },
-   { "(SCHU)", "", 073672 },
-   { "TOPMEM", "", 077677 },
-   { "",       "", -1     }
-}; 
+#include "asmsymbols.h"
 
 /***********************************************************************
 * tabpos - Return TRUE if col is a tab stop.
@@ -425,7 +140,6 @@ tabpos (int col, int tabs[])
 
 static void
 detabit (int tabs[], FILE *ifd, FILE *ofd)
-         
 {
    int c, col;
  
@@ -448,6 +162,7 @@ detabit (int tabs[], FILE *ifd, FILE *ofd)
          case '\n' :
             fputc (c, ofd);
             col = 0;
+	    reccnt++;
             break;
 
          default: 
@@ -458,7 +173,7 @@ detabit (int tabs[], FILE *ifd, FILE *ofd)
 
    } /* while */
 
-} /* detab */
+} /* detabit */
 
 /***********************************************************************
 * alldig - Check if all digits.
@@ -545,23 +260,29 @@ settab (int tabs[], int argc, char *argv[])
 * detab - Detab source.
 ***********************************************************************/
 
-static int
-detab (FILE *ifd, FILE *ofd) 
+int
+detab (FILE *ifd, FILE *ofd, char *type) 
 {
    int tabs[BUFSIZ];
    char *tabstops[] = { "as", "7", "15", "+8", "\0" };
   
 #ifdef DEBUG
-   printf ("detab: Entered: infile = %s, outfile = %s\n",
+   fprintf (stderr, "detab: Entered: infile = %s, outfile = %s\n",
 	    infile, outfile);
 #endif
 
+   if (verbose)
+      fprintf (stderr, "asm7090: Detab %s file \n", type);
+   reccnt = 0;
+
    /* set initial tab stops */
 
    settab (tabs, 4, tabstops);
 
    detabit (tabs, ifd, ofd);
 
+   if (verbose)
+      fprintf (stderr, "asm7090: File contains %d records\n", reccnt);
    return (0);
 
 } /* detab */
@@ -578,7 +299,8 @@ definemonsyms (int fapselect)
    int oldline;
 
 #ifdef DEBUGMONSYM
-   printf ("definemonsyms: fapmode = %d, absmod = %d, fapselect = %d\n",
+   fprintf (stderr,
+	 "definemonsyms: fapmode = %d, absmod = %d, fapselect = %d\n",
 	    fapmode, absmod, fapselect);
 #endif
    if (!fapmode && !absmod) return;
@@ -616,7 +338,7 @@ definemonsyms (int fapselect)
       if (s)
       {
 	 s->value = ss[i].val;
-	 s->relocatable = FALSE;
+	 s->flags = 0;
       }
    }
    linenum = oldline;
@@ -653,7 +375,7 @@ definejobsyms (void)
 	       != NULL)
 	 {
 	    s->value = jobdefs[i].val;
-	    s->relocatable = FALSE;
+	    s->flags = 0;
 	 }
       }
       linenum = oldline;
@@ -680,16 +402,16 @@ main (int argc, char **argv)
    int done;
    int listmode;
    int status = 0;
-   char mkname[32];
-   char tname0[64];
-   char tname1[64];
+   int tmpdes0, tmpdes1;
+   char tname0[MAXFILENAMELEN+2];
+   char tname1[MAXFILENAMELEN+2];
   
 #ifdef DEBUG
-   printf ("asm7090: Entered:\n");
-   printf ("args:\n");
+   fprintf (stderr, "asm7090: Entered:\n");
+   fprintf (stderr, "args:\n");
    for (i = 1; i < argc; i++)
    {
-      printf ("   arg[%2d] = %s\n", i, argv[i]);
+      fprintf (stderr, "   arg[%2d] = %s\n", i, argv[i]);
    }
 #endif
 
@@ -697,6 +419,8 @@ main (int argc, char **argv)
    freexrefs = NULL;
    freeops = NULL;
 
+   termparn = TRUE;
+
    gblabsmod = FALSE;
    gbljobsym = FALSE;
    gblmonsym = FALSE;
@@ -705,11 +429,14 @@ main (int argc, char **argv)
    genxref = FALSE;
    fapmode = FALSE;
    listmode = FALSE;
-   termparn = TRUE;
    widemode = FALSE;
+   sequcheck = FALSE;
+   punchsymtable = FALSE;
+   ctsscommon = FALSE;
 
    filecnt = 0;
    p1erralloc = 0;
+   incldircnt = 0;
    cpumodel = 7090;
 
    deckname[0] = '\0';
@@ -731,23 +458,39 @@ main (int argc, char **argv)
 	    gblabsmod = TRUE;
 	    break;
 
+	 case 'C': /* Relocate CTSS Common mode */
+	    ctsscommon = TRUE;
+	    break;
+
 	 case 'c': /* CPU model */
 	    i++;
-	    cpumodel = atoi(argv[i]);
-	    switch (cpumodel)
+	    if (i >= argc) goto USAGE;
+	    if (!strcmp (argv[i], "CTSS") || !strcmp (argv[i], "ctss"))
 	    {
-	    case 704:
-	    case 709:
-	    case 7090:
-	    case 7094:
-	       break;
-	    default:
-	       goto USAGE;
+	       /* For CTSS we add model 7096 and run in FAP mode */
+	       cpumodel = 7096;
+	       fapmode = TRUE;
+	       termparn = FALSE;
+	    }
+	    else
+	    {
+	       cpumodel = atoi(argv[i]);
+	       switch (cpumodel)
+	       {
+	       case 704:
+	       case 709:
+	       case 7090:
+	       case 7094:
+		  break;
+	       default:
+		  goto USAGE;
+	       }
 	    }
 	    break;
 
 	 case 'd': /* Deckname */
 	    i++;
+	    if (i >= argc) goto USAGE;
 	    if (strlen (argv[i]) > MAXSYMLEN) argv[i][MAXSYMLEN] = '\0';
 	    strcpy (deckname, argv[i]);
 	    break;
@@ -757,12 +500,19 @@ main (int argc, char **argv)
 	    termparn = FALSE;
 	    break;
 
+	 case 'I': /* Include directory */
+	    i++;
+	    if (i >= argc) goto USAGE;
+	    incldir[incldircnt++] = argv[i];
+	    break;
+
 	 case 'j': /* JOBSYM */
 	    gbljobsym = TRUE;
 	    break;
 
 	 case 'l': /* Listing file */
 	    i++;
+	    if (i >= argc) goto USAGE;
 	    lstfile = argv[i];
 	    listmode = TRUE;
 	    break;
@@ -773,6 +523,7 @@ main (int argc, char **argv)
 
          case 'o': /* Output file */
             i++;
+	    if (i >= argc) goto USAGE;
             outfile = argv[i];
             break;
 
@@ -784,8 +535,17 @@ main (int argc, char **argv)
 	    gblabsmod = FALSE;
 	    break;
 
+	 case 's': /* punch symbol table */
+	    punchsymtable = TRUE;
+	    break;
+
+	 case 'S': /* Sequence check */
+	    sequcheck = TRUE;
+	    break;
+
 	 case 't': /* Title */
 	    i++;
+	    if (i >= argc) goto USAGE;
 	    if (strlen (argv[i]) > WIDETITLELEN) argv[i][WIDETITLELEN] = '\0';
 	    strcpy (titlebuf, argv[i]);
 	    break;
@@ -806,23 +566,25 @@ main (int argc, char **argv)
       USAGE:
 	    fprintf (stderr, "asm7090 - version %s\n", VERSION);
 	    fprintf (stderr,
-		     "usage: asm7090 [-options] -o file.obj file.asm\n");
+		 "usage: asm7090 [-options] -o file.obj file.asm\n");
             fprintf (stderr, " options:\n");
-	    fprintf (stderr, "    -a           - Absolute assembly\n");
+	    fprintf (stderr, "    -a         - Absolute assembly\n");
+	    fprintf (stderr, "    -C         - Relocate CTSS Program Common\n");
 	    fprintf (stderr,
-		     "    -c model     - CPU model (704, 709, 7090, 7094)\n");
-	    fprintf (stderr, "    -d deck      - Deckname for output\n");
-	    fprintf (stderr, "    -f           - FAP assembly mode\n");
-	    fprintf (stderr, "    -j           - JOBSYM mode\n");
-	    fprintf (stderr,
-		     "    -l listfile  - Generate listing to listfile\n");
-	    fprintf (stderr, "    -m           - MONSYM mode\n");
-	    fprintf (stderr, "    -o outfile   - Object file name\n");
-	    fprintf (stderr, "    -p           - ParensOK == ()OK\n");
-	    fprintf (stderr, "    -r           - ADDREL mode\n");
-	    fprintf (stderr, "    -t title     - Title for listing\n");
-	    fprintf (stderr, "    -w           - Generate wide listing\n");
-	    fprintf (stderr, "    -x           - Generate cross reference\n");
+		 "    -c model   - CPU model (704, 709, 7090, 7094, CTSS)\n");
+	    fprintf (stderr, "    -d deck    - Deckname for output\n");
+	    fprintf (stderr, "    -f         - FAP assembly mode\n");
+	    fprintf (stderr, "    -I incldir - Include directory for INSERT\n");
+	    fprintf (stderr, "    -j         - JOBSYM mode\n");
+	    fprintf (stderr, "    -l lstfile - Generate listing to lstfile\n");
+	    fprintf (stderr, "    -m         - MONSYM mode\n");
+	    fprintf (stderr, "    -o outfile - Object file name\n");
+	    fprintf (stderr, "    -p         - ParensOK == ()OK\n");
+	    fprintf (stderr, "    -r         - ADDREL mode\n");
+	    fprintf (stderr, "    -s         - Punch symbol table\n");
+	    fprintf (stderr, "    -t title   - Title for listing\n");
+	    fprintf (stderr, "    -w         - Generate wide listing\n");
+	    fprintf (stderr, "    -x         - Generate cross reference\n");
 	    return (ABORT);
          }
       }
@@ -836,7 +598,7 @@ main (int argc, char **argv)
    }
    if (!widemode) titlebuf[NARROWTITLELEN] = '\0';
 
-   if (!infile && !outfile) goto USAGE;
+   if (!infile || !outfile) goto USAGE;
 
    if (!strcmp (infile, outfile))
    {
@@ -858,10 +620,10 @@ main (int argc, char **argv)
    }
 
 #ifdef DEBUG
-   printf (" infile = %s\n", infile);
-   printf (" outfile = %s\n", outfile);
+   fprintf (stderr, " infile = %s\n", infile);
+   fprintf (stderr, " outfile = %s\n", outfile);
    if (listmode)
-      printf (" lstfile = %s\n", lstfile);
+      fprintf (stderr, " lstfile = %s\n", lstfile);
 #endif
 
    /*
@@ -892,44 +654,27 @@ main (int argc, char **argv)
    ** Create a temp files for intermediate use.
    */
 
-   strcpy (mkname, TEMPSPEC);
-   if ((bp = (char *)mktemp (mkname)) == NULL)
+#if defined(WIN32)
+   sprintf (tname0, "a0%s", TEMPSPEC);
+#else
+   sprintf (tname0, "/tmp/a0%s", TEMPSPEC);
+#endif
+   if ((tmpdes0 = mkstemp (tname0)) < 0)
    {
-      perror ("asm7090: Can't mktemp");
+      perror ("asm7090: Can't mkstemp");
       return (ABORT);
    }
 
-#ifdef WIN32
-   sprintf (tname0, "%s1.s", bp);
-#else
-   sprintf (tname0, "/tmp/%s1.s", bp);
-#endif
 #ifdef DEBUGTMPFILE
-   printf ("asm7090: tmp0 = %s\n", tname0);
+   fprintf (stderr, "asm7090: tmp0 = %s\n", tname0);
 #endif
 
-   if ((tmpfd0 = fopen (tname0, "w+")) == NULL)
+   if ((tmpfd0 = fdopen (tmpdes0, "w+")) == NULL)
    {
       perror ("asm7090: Can't open temporary file");
       exit (1);
    }
 
-   strcpy (mkname, TEMPSPEC);
-   if ((bp = (char *)mktemp (mkname)) == NULL)
-   {
-      perror ("asm7090: Can't mktemp");
-      return (ABORT);
-   }
-
-#ifdef WIN32
-   sprintf (tname1, "%s2.s", bp);
-#else
-   sprintf (tname1, "/tmp/%s2.s", bp);
-#endif
-#ifdef DEBUGTMPFILE
-   printf ("asm7090: tmp1 = %s\n", tname1);
-#endif
-
    /*
    ** Get current date/time.
    */
@@ -939,11 +684,16 @@ main (int argc, char **argv)
    strcpy (datebuf, ctime(&curtime));
    *strchr (datebuf, '\n') = '\0';
 
+#ifdef DEBUGPARN
+   fprintf (stderr, "termparnI = %s\n", termparn ? "TRUE" : "FALSE");
+#endif
+
    /*
    ** Detab the source, so we know where we are on a line.
    */
 
-   status = detab (infd, tmpfd0);
+   if (verbose) fprintf (stderr, "\n");
+   status = detab (infd, tmpfd0, "source");
 
    /*
    ** Rewind the input.
@@ -983,12 +733,20 @@ main (int argc, char **argv)
       }
 
       addextsym = NULL;
+      rmthead = NULL;
+      rmttail = NULL;
+
       rboolexpr = FALSE;
       lboolexpr = FALSE;
       mondefsset = FALSE;
       jobdefsset = FALSE;
       ifapdefsset = FALSE;
       ffapdefsset = FALSE;
+      genlnkdir = FALSE;
+      nolnkdir = FALSE;
+      bcore = FALSE;
+      commonused = FALSE;
+
       exprtype = ADDREXPR;
 
       absmod = gblabsmod;
@@ -1013,6 +771,7 @@ main (int argc, char **argv)
       opdefcount = 0;
       headcount = 0;
       errcount = 0;
+      warncount = 0;
       errnum = 0;
       linenum = 0;
       p1errcnt = 0;
@@ -1021,7 +780,10 @@ main (int argc, char **argv)
       qualindex = 0;
       litorigin = 0;
       litpool = 0;
-      qualindex = 0;
+      filenum = 0;
+      fileseq = 0;
+
+      commonctr = FAPCOMMONSTART;
 
       ttlbuf[0] = '\0';
       lbl[0] = '\0';
@@ -1032,6 +794,7 @@ main (int argc, char **argv)
       begintable[0].chain = 0;
       begintable[0].bvalue = 0;
       begintable[0].value = 0;
+      pscanbuf = inbuf;
 
       if (verbose)
 	 fprintf (stderr, "\nasm7090: Processing file %d\n", filecnt+1);
@@ -1050,11 +813,41 @@ main (int argc, char **argv)
       if (jobsym)
 	 definejobsyms ();
 
+      /*
+      ** If CTSS mode, define XBUILD
+      */
+
+      if (cpumodel == 7096)
+      {
+	 SymNode *s;
+
+	 if (s = symlookup ("XBUILD", "", TRUE, TRUE))
+	 {
+	    s->value = 1;
+	    s->flags = SETVAR;
+	 }
+      }
+
       /*
       ** Open the intermediate temp file
       */
 
-      if ((tmpfd1 = fopen (tname1, "w+b")) == NULL)
+#if defined(WIN32)
+      sprintf (tname1, "a1%s", TEMPSPEC);
+#else
+      sprintf (tname1, "/tmp/a1%s", TEMPSPEC);
+#endif
+      if ((tmpdes1 = mkstemp (tname1)) < 0)
+      {
+	 perror ("asm7090: Can't mkstemp");
+	 return (ABORT);
+      }
+
+#ifdef DEBUGTMPFILE
+   fprintf (stderr, "asm7090: tmp1 = %s\n", tname1);
+#endif
+
+      if ((tmpfd1 = fdopen (tmpdes1, "w+b")) == NULL)
       {
 	 perror ("asm7090: Can't open temporary file");
 	 exit (1);
@@ -1069,16 +862,32 @@ main (int argc, char **argv)
 	 fprintf (stderr, "asm7090: Calling pass %03o\n", inpass);
 
       status = asmpass0 (tmpfd0, tmpfd1);
-      if (status == 1)
+#ifdef DEBUGPARN
+      fprintf (stderr, "termparn0 = %s\n", termparn ? "TRUE" : "FALSE");
+#endif
+      if (status)
       {
 	 if (verbose)
 	    fprintf  (stderr, "asm7090: File %d: EOF\n",
 		     filecnt+1);
 	 if (tmpfd1)
+	 {
 	    fclose (tmpfd1);
+	    unlink (tname1);
+	 }
 	 tmpfd1 = NULL;
-	 status = 0;
          done = TRUE;
+	 if (status == 2)
+	 {
+	    if (listmode)
+	       fprintf (lstfd, "\nNo END record\n");
+	    filecnt++;
+	    goto PRINT_ERROR;
+	 }
+	 else
+	 {
+	    status = 0;
+	 }
 	 break;
       }
       if (filecnt++ > 0 && listmode)
@@ -1093,10 +902,13 @@ main (int argc, char **argv)
 	 fprintf (stderr, "asm7090: Calling pass %03o\n", inpass);
 
       status = asmpass1 (tmpfd1, TRUE);
+#ifdef DEBUGPARN
+      fprintf (stderr, "termparn1 = %s\n", termparn ? "TRUE" : "FALSE");
+#endif
 
       /* 
       ** Call pass 1 again to scan the source and assign storage.
-      ** We have to do this twice because MAP allows forward decls of
+      ** We have to do this again because MAP allows forward decls of
       ** variables that affect storage. Also, USE/BEGIN and COMMON.
       */
 
@@ -1125,13 +937,19 @@ main (int argc, char **argv)
 	 fprintf (stderr, "asm7090: Calling pass %03o\n", inpass);
 
       status = asmpass2 (tmpfd1, outfd, listmode, lstfd);
+#ifdef DEBUGPARN
+      fprintf (stderr, "termparn2 = %s\n", termparn ? "TRUE" : "FALSE");
+#endif
 
       /*
       ** Close the intermediate file
       */
 
       if (tmpfd1)
+      {
 	 fclose (tmpfd1);
+	 unlink (tname1);
+      }
       tmpfd1 = NULL;
 
       /*
@@ -1178,18 +996,30 @@ main (int argc, char **argv)
 	    freeopd (opdefcode[i]);
       }
 
+      while (rmthead)
+      {
+         RMTSeq *lclnext;
+
+	 lclnext = rmthead->next;
+	 DBG_MEMREL (rmthead);
+	 rmthead = lclnext;
+      }
+
       /*
       ** Print assembly error count.
       */
 
+   PRINT_ERROR:
       if (listmode)
       {
-	 fprintf (lstfd, "\n%d Errors in assembly\n", errcount);
+	 fprintf (lstfd, "\n%d Warnings in assembly\n", warncount);
+	 fprintf (lstfd, "%d Errors in assembly\n", errcount);
       }
-      if (errcount || verbose)
+      if (errcount || warncount || verbose)
       {
-	 fprintf (stderr, "asm7090: File %d: %d Errors in assembly\n",
-		  filecnt, errcount);
+	 fprintf (stderr,
+		  "asm7090: File %d: %d Errors, %d Warnings in assembly\n",
+		  filecnt, errcount, warncount);
       }
 
       titlebuf[0] = '\0';
@@ -1207,17 +1037,13 @@ main (int argc, char **argv)
    if (outfd)
       fclose (outfd);
    if (tmpfd0)
+   {
       fclose (tmpfd0);
+      unlink (tname0);
+   }
    if (listmode && lstfd)
       fclose (lstfd);
 
-   /*
-   ** Delete tmp files
-   */
-
-   unlink (tname0);
-   unlink (tname1);
-
    if (status != 0)
       unlink (outfile);
 
