@@ -5,6 +5,8 @@
 * Changes:
 *   05/21/03   DGP   Original.
 *   03/18/05   DGP   Added '?' and '!' characters.
+*   06/10/05   DGP   Added 12 bit tables.
+*   06/25/10   DGP   Added CTSS characters.
 *	
 ***********************************************************************/
 
@@ -32,7 +34,7 @@ static unsigned char tobcd[128] =
  /*30   0    1    2    3    4    5    6    7 */
       000, 001, 002, 003, 004, 005, 006, 007,
  /*38   8    9    :    ;    <    =    >    ? */
-      010, 011, 060, 060, 060, 013, 060, 032,
+      010, 011, 035, 060, 060, 013, 060, 032,
  /*40   @    A    B    C    D    E    F    G */
       060, 021, 022, 023, 024, 025, 026, 027,
  /*48   H    I    J    K    L    M    N    O */
@@ -93,7 +95,7 @@ static unsigned char tobcd[256] =
  /*70                                        */
       060, 060, 060, 060, 060, 060, 060, 060,
  /*78        `    :    #    @    '    =    " */
-      060, 060, 060, 060, 060, 014, 013, 060,
+      060, 060, 035, 060, 060, 014, 013, 060,
  /*80        a    b    c    d    e    f    g */
       060, 021, 022, 023, 024, 025, 026, 027,
  /*88   h    i         {                     */
@@ -129,3 +131,28 @@ static unsigned char tobcd[256] =
 };
 
 #endif /* EBCDIC */
+
+/*
+** 12BIT tables.
+*/
+
+uint8 bit12[64] =
+{
+/* 00  '0',  '1',  '2',  '3',  '4',  '5',  '6',  '7', */
+       0000, 0001, 0002, 0003, 0004, 0005, 0006, 0007,
+/* 10  '8',  '9',  ' ',  '=',  '\'', ' ',  ' ',  ' ', */
+       0010, 0011, 0012, 0013, 0014, 0015, 0016, 0017,
+/* 20  '+',  'A',  'B',  'C',  'D',  'E',  'F',  'G', */
+       0020, 0121, 0122, 0123, 0124, 0125, 0126, 0127,
+/* 30  'H',  'I',  '?',  '.',  ')',  ':',  ' ',  ' ', */
+       0130, 0131, 0032, 0033, 0034, 0035, 0036, 0037,
+/* 40  '-',  'J',  'K',  'L',  'M',  'N',  'O',  'P', */
+       0040, 0141, 0142, 0143, 0144, 0145, 0146, 0147,
+/* 50  'Q',  'R',  '!',  '$',  '*',  ' ',  ' ',  ' ', */
+       0150, 0151, 0052, 0053, 0054, 0055, 0056, 0057,
+/* 60  ' ',  '/',  'S',  'T',  'U',  'V',  'W',  'X', */
+       0060, 0061, 0162, 0163, 0164, 0165, 0166, 0167,
+/* 70  'Y',  'Z',  ' ',  ',',  '(',  ' ',  ' ',  ' '  */
+       0170, 0171, 0072, 0073, 0074, 0075, 0076, 0077
+};
+
