@@ -17,6 +17,8 @@
 *   12/02/11   DGP   Added LINK opcode.
 *   06/20/12   DGP   Changed MAXMACROS for z/OS (USS).
 *   06/21/12   DGP   More realistic sizes.
+*   10/21/21   DGP   Boosted FIELD sizes for long MACRO operands and ETC.
+*   03/25/22   DGP   Added MAXLSYMLEN for literals.
 *	
 ***********************************************************************/
 
@@ -36,7 +38,7 @@
 #define FALSE 0
 #endif
 
-#define VERSION "2.2.29"
+#define VERSION "2.3.2"
 
 #define IBSYSSYM    '$'		/* Begins an IBSYS control card */
 #define COMMENTSYM  '*'		/* Begins a comment */
@@ -55,7 +57,7 @@
 
 #define MAXDUPLINES 200		/* Max DUP lines */
 #define MAXMACROLINES 200	/* Max MACRO lines */
-#define MAXETCLINES 10		/* Max ETC lines */
+#define MAXETCLINES 30		/* Max ETC lines */
 
 #define MAXSRCLINE 90		/* Max length of a single source line */
 #define MAXLINE MAXSRCLINE*MAXETCLINES /* Max length of source line, w/ ETCs  */
@@ -67,7 +69,8 @@
 #define MAXLBLLEN 8		/* Max length of LBL */
 #define MAXMACNAMELEN 6		/* Max length of MACRO name */
 #define MAXSYMLEN 6		/* Max length of symbol */
-#define MAXFIELDSIZE 80		/* Max length of a MACRO field */
+#define MAXFIELDSIZE 800	/* Max length of a MACRO field */
+#define MAXLSYMLEN 80		/* Max length of literal symbol */
 
 #define LINESPAGE 55		/* Max lines on a listing page */
 
@@ -281,8 +284,8 @@ typedef struct Op_DefCode
 typedef struct Mac_Lines
 {
    struct Mac_Lines *next;
-   char label[MAXFIELDSIZE];
-   char opcode[MAXFIELDSIZE];
+   char label[MAXSYMLEN+2];
+   char opcode[MAXSYMLEN+2];
    char operand[MAXLINE];
 } MacLines;
 
@@ -322,8 +325,8 @@ typedef struct Sym_Node
    XrefNode *xref_head;
    XrefNode *xref_tail;
    struct Sym_Node *vsym;
-   char symbol[MAXFIELDSIZE];
-   char qualifier[MAXFIELDSIZE];
+   char symbol[MAXLSYMLEN+2];
+   char qualifier[MAXLSYMLEN+2];
    uint32 flags;
    int value;
    int locctr;
