@@ -51,6 +51,7 @@
 *   12/16/10   DGP   Added a unified source read function readsource().
 *   12/24/10   DGP   Correct page count to eject new page.
 *   05/06/13   DGP   Fixed QUAL processing.
+*   10/29/21   DGP   Truncate long listing lines.
 *	
 ***********************************************************************/
 
@@ -3654,6 +3655,10 @@ asmpass2 (FILE *tmpfd, FILE *outfd, int lstmode, FILE *lstfd)
 	       break;
 	 }
       }
+      else if (strlen(lstbuf) > 81)
+      {
+	 strcpy (&lstbuf[80], " ...\n");
+      }
 
       /*
       ** If a IBSYS control card, print it.
