@@ -32,6 +32,7 @@
 *   11/11/11   DGP   Allow INSERT files to have "inc" or "fap" extension.
 *   11/16/11   DGP   In tokscan return qualifer with token.
 *   06/19/12   DGP   Fix scantok bug.
+*   06/06/22   DGP   Fix qual count bug in symlocate.
 *	
 ***********************************************************************/
 
@@ -1314,7 +1315,8 @@ symlocate (char *sym, int *count)
 	 */
 
 	 lo = mid;
-	 while (!strcmp (symbols[mid]->symbol, symbols[lo-1]->symbol) &&
+	 while ((lo > 0) &&
+	 	!strcmp (symbols[mid]->symbol, symbols[lo-1]->symbol) &&
 	        strcmp (symbols[mid]->qualifier, symbols[lo-1]->qualifier))
 	 {
 #ifdef DEBUGSYMLOC
@@ -1323,7 +1325,7 @@ symlocate (char *sym, int *count)
 #endif
 	    lo--;
 	 }
-	 up = mid;
+	 up = mid + 1;
 	 while (!strcmp (symbols[mid]->symbol, symbols[up-1]->symbol) &&
 	        strcmp (symbols[mid]->qualifier, symbols[up-1]->qualifier))
 	 {
